@@ -1,3 +1,4 @@
+/* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 2; tab-width: 2 -*- */
 /*
  * Copyright © 1998, 2003 Jonathan Blandford <jrb@alum.mit.edu>
  * Copyright © 2003 Callum McKenzie <callum@physics.otago.ac.nz>
@@ -63,8 +64,8 @@
 
 #define AISLERIOT_WINDOW_GET_PRIVATE(window)(G_TYPE_INSTANCE_GET_PRIVATE ((window), AISLERIOT_TYPE_WINDOW, AisleriotWindowPrivate))
 
-#define MIN_WIDTH 600
-#define MIN_HEIGHT 400
+#define MIN_WIDTH 800
+#define MIN_HEIGHT 600
 
 #define MAIN_MENU_UI_PATH       "/MainMenu"
 #define RECENT_GAMES_MENU_PATH  MAIN_MENU_UI_PATH "/GameMenu/RecentMenu"
@@ -142,7 +143,6 @@ struct _AisleriotWindowPrivate
   guint load_idle_id;
 
   guint changing_game_type : 1;
-  guint freecell_mode : 1;
   guint toolbar_visible : 1;
   guint statusbar_visible : 1;
   guint fullscreen : 1;
@@ -153,11 +153,6 @@ enum {
   OPTION_RADIOMENU
 };
 
-enum {
-  PROP_0,
-  PROP_FREECELL_MODE
-};
-
 /* Game Over dialogue */
 
 enum {
@@ -312,11 +307,10 @@ stats_dialog_response_cb (GtkWidget *widget,
   gtk_widget_destroy (widget);
 }
 
-/* action callbacks */
+/* action methods */
 
-static void
-new_game_cb (GtkAction *action,
-             AisleriotWindow *window)
+void
+aisleriot_window_new_game (AisleriotWindow *window)
 {
   AisleriotWindowPrivate *priv = window->priv;
 
@@ -325,42 +319,53 @@ new_game_cb (GtkAction *action,
   gtk_widget_grab_focus (GTK_WIDGET (priv->board));
 }
 
-static void
-undo_cb (GtkAction *action,
-         AisleriotWindow *window)
+void
+aisleriot_window_change_game (AisleriotWindow *window)
 {
   AisleriotWindowPrivate *priv = window->priv;
 
-  /* If a move is in progress, cancel it before changing the game! */
-#ifdef HAVE_CLUTTER
-  aisleriot_board_abort_move (AISLERIOT_BOARD (priv->board_actor));
-#else
-  aisleriot_board_abort_move (priv->board);
-#endif
+  if (priv->game_choice_dialog) {
+    gtk_window_present (GTK_WINDOW (priv->game_choice_dialog));
+    return;
+  }
 
-  aisleriot_game_undo_move (priv->game);
+  priv->game_choice_dialog = ar_game_chooser_new (window);
+  g_signal_connect (priv->game_choice_dialog, "destroy",
+                    G_CALLBACK (gtk_widget_destroyed), &priv->game_choice_dialog);
+
+  gtk_window_present (GTK_WINDOW (priv->game_choice_dialog));
 }
 
-static void
-redo_cb (GtkAction *action,
-         AisleriotWindow *window)
+void
+aisleriot_window_show_set_fullscreen (AisleriotWindow *window,
+                                      gboolean         active)
 {
-  AisleriotWindowPrivate *priv = window->priv;
-
-#ifdef HAVE_CLUTTER
-  aisleriot_board_abort_move (AISLERIOT_BOARD (priv->board_actor));
-#else
-  aisleriot_board_abort_move (priv->board);
-#endif
-
-  aisleriot_game_redo_move (priv->game);
 }
 
-static void
-help_about_cb (GtkAction *action,
-               AisleriotWindow *window)
+void
+aisleriot_window_show_statistics_dialog (AisleriotWindow *window)
 {
   AisleriotWindowPrivate *priv = window->priv;
+
+  if (!priv->stats_dialog) {
+    priv->stats_dialog = aisleriot_stats_dialog_new ();
+    gtk_window_set_transient_for (GTK_WINDOW (priv->stats_dialog),
+                                  GTK_WINDOW (window));
+
+    g_signal_connect (priv->stats_dialog, "response",
+                      G_CALLBACK (stats_dialog_response_cb), window);
+    g_signal_connect (priv->stats_dialog, "destroy",
+                      G_CALLBACK (gtk_widget_destroyed), &priv->stats_dialog);
+  }
+
+  update_statistics_display (window);
+
+  gtk_window_present (GTK_WINDOW (priv->stats_dialog));
+}
+
+void
+aisleriot_window_show_about_dialog (AisleriotWindow * window)
+{
   const char *authors[] = {
     _("Main game:"),
     "Jonathan Blandford <jrb@redhat.com>",
@@ -406,30 +411,25 @@ help_about_cb (GtkAction *action,
 
   char *licence;
 
-  licence = ar_get_licence (priv->freecell_mode ? _("FreeCell Solitaire") : ("AisleRiot"));
+  licence = ar_get_licence ("AisleRiot");
 
   gtk_show_about_dialog (GTK_WINDOW (window),
                          "program-name",
-                            priv->freecell_mode ? _("FreeCell Solitaire")
-                                                : _("AisleRiot"),
+                         _("AisleRiot"),
                          "version", VERSION,
-                         "title", priv->freecell_mode ? _("About FreeCell Solitaire")
-                                                      : _("About AisleRiot"),
+                         "title", _("About AisleRiot"),
                          "comments",
-                         priv->freecell_mode ?
-                           NULL :
-                           _("AisleRiot provides a rule-based solitaire "
-                             "card engine that allows many different "
-                             "games to be played."),
+                         _("AisleRiot provides a rule-based solitaire "
+                           "card engine that allows many different "
+                           "games to be played."),
                          "copyright", "Copyright © 1998-2006 Jonathan Blandford\n"
-                                      "Copyright © 2007, 2008, 2009, 2010, 2011, 2012 Christian Persch",
+                                      "Copyright © 2007, 2008, 2009, 2010, 2011, 2012, 2013 Christian Persch",
                          "license", licence,
                          "authors", authors,
                          "artists", artists,
                          "documenters", documenters,
                          "translator-credits", _("translator-credits"),
-                         "logo-icon-name", priv->freecell_mode ? "gnome-freecell"
-                                                               : "gnome-aisleriot",
+                         "logo-icon-name", "gnome-aisleriot",
                          "website", "http://www.gnome.org/projects/gnome-games/",
                          "website-label", _("GNOME Games web site"),
                          "wrap-license", TRUE,
@@ -437,6 +437,53 @@ help_about_cb (GtkAction *action,
   g_free (licence);
 }
 
+/* action callbacks */
+
+static void
+new_game_cb (GtkAction *action,
+             AisleriotWindow *window)
+{
+  aisleriot_window_new_game (window);
+}
+
+static void
+undo_cb (GtkAction *action,
+         AisleriotWindow *window)
+{
+  AisleriotWindowPrivate *priv = window->priv;
+
+  /* If a move is in progress, cancel it before changing the game! */
+#ifdef HAVE_CLUTTER
+  aisleriot_board_abort_move (AISLERIOT_BOARD (priv->board_actor));
+#else
+  aisleriot_board_abort_move (priv->board);
+#endif
+
+  aisleriot_game_undo_move (priv->game);
+}
+
+static void
+redo_cb (GtkAction *action,
+         AisleriotWindow *window)
+{
+  AisleriotWindowPrivate *priv = window->priv;
+
+#ifdef HAVE_CLUTTER
+  aisleriot_board_abort_move (AISLERIOT_BOARD (priv->board_actor));
+#else
+  aisleriot_board_abort_move (priv->board);
+#endif
+
+  aisleriot_game_redo_move (priv->game);
+}
+
+static void
+help_about_cb (GtkAction *action,
+               AisleriotWindow *window)
+{
+  aisleriot_window_show_about_dialog (window);
+}
+
 static void
 restart_game (GtkAction *action,
               AisleriotWindow *window)
@@ -450,18 +497,7 @@ static void
 select_game_cb (GtkAction *action,
                 AisleriotWindow *window)
 {
-  AisleriotWindowPrivate *priv = window->priv;
-
-  if (priv->game_choice_dialog) {
-    gtk_window_present (GTK_WINDOW (priv->game_choice_dialog));
-    return;
-  }
-
-  priv->game_choice_dialog = ar_game_chooser_new (window);
-  g_signal_connect (priv->game_choice_dialog, "destroy",
-                    G_CALLBACK (gtk_widget_destroyed), &priv->game_choice_dialog);
-
-  gtk_window_present (GTK_WINDOW (priv->game_choice_dialog));
+  aisleriot_window_change_game (window);
 }
 
 static void
@@ -493,22 +529,7 @@ static void
 statistics_cb (GtkAction *action,
                AisleriotWindow *window)
 {
-  AisleriotWindowPrivate *priv = window->priv;
-
-  if (!priv->stats_dialog) {
-    priv->stats_dialog = aisleriot_stats_dialog_new ();
-    gtk_window_set_transient_for (GTK_WINDOW (priv->stats_dialog),
-                                  GTK_WINDOW (window));
-
-    g_signal_connect (priv->stats_dialog, "response",
-                      G_CALLBACK (stats_dialog_response_cb), window);
-    g_signal_connect (priv->stats_dialog, "destroy",
-                      G_CALLBACK (gtk_widget_destroyed), &priv->stats_dialog);
-  }
-
-  update_statistics_display (window);
-
-  gtk_window_present (GTK_WINDOW (priv->stats_dialog));
+  aisleriot_window_show_statistics_dialog (window);
 }
 
 static void
@@ -939,12 +960,11 @@ clickmove_toggle_cb (GtkToggleAction *action,
   ar_conf_set_boolean (NULL, aisleriot_conf_get_key (CONF_CLICK_TO_MOVE), click_to_move);
 }
 
-#ifdef ENABLE_SOUND
-
 static void
 sound_toggle_cb (GtkToggleAction *action,
                  AisleriotWindow *window)
 {
+#ifdef ENABLE_SOUND
   gboolean sound_enabled;
 
   sound_enabled = gtk_toggle_action_get_active (action);
@@ -952,16 +972,14 @@ sound_toggle_cb (GtkToggleAction *action,
   ar_sound_enable (sound_enabled);
   
   ar_conf_set_boolean (NULL, aisleriot_conf_get_key (CONF_SOUND), sound_enabled);
-}
-
 #endif /* ENABLE_SOUND */
-
-#ifdef HAVE_CLUTTER
+}
 
 static void
 animations_toggle_cb (GtkToggleAction *action,
                       AisleriotWindow *window)
 {
+#ifdef HAVE_CLUTTER
   AisleriotWindowPrivate *priv = window->priv;
   gboolean enabled;
 
@@ -970,9 +988,8 @@ animations_toggle_cb (GtkToggleAction *action,
   ar_style_set_enable_animations (priv->board_style, enabled);
   
   ar_conf_set_boolean (NULL, aisleriot_conf_get_key (CONF_ANIMATIONS), enabled);
-}
-
 #endif /* HAVE_CLUTTER */
+}
 
 static void
 show_hint_cb (GtkAction *action,
@@ -1197,17 +1214,12 @@ static void
 add_recently_played_game (AisleriotWindow *window,
                           const char *game_module)
 {
-  AisleriotWindowPrivate *priv = window->priv;
   char **recent_games, **new_recent;
   gsize i, n_recent = 0, n_new_recent = 0;
 
   if (!game_module)
     return;
 
-  /* Don't store the game type in freecell mode */
-  if (priv->freecell_mode)
-    return;
-
   recent_games = ar_conf_get_string_list (NULL, aisleriot_conf_get_key (CONF_RECENT_GAMES), &n_recent, NULL);
 
   if (recent_games == NULL) {
@@ -1603,10 +1615,7 @@ game_type_changed_cb (AisleriotGame *game,
   g_object_set (priv->action[ACTION_HELP_GAME], "label", game_name, NULL);
   g_object_set (priv->action[ACTION_OPTIONS_MENU], "label", game_name, NULL);
 
-  /* In freecell mode, we've already set the title to something different */
-  if (!priv->freecell_mode) {
-    gtk_window_set_title (GTK_WINDOW (window), game_name);
-  }
+  gtk_window_set_title (GTK_WINDOW (window), game_name);
 
   g_free (game_name);
 
@@ -1771,13 +1780,12 @@ game_exception_cb (AisleriotGame *game,
   gtk_widget_show (dialog);
 }
 
-#if defined(HAVE_CLUTTER) || defined(ENABLE_SOUND)
-
 static void
 settings_changed_cb (GtkSettings *settings,
                      GParamSpec *pspec,
                      AisleriotWindow *window)
 {
+#if defined(HAVE_CLUTTER) || defined(ENABLE_SOUND)
   AisleriotWindowPrivate *priv = window->priv;
   GtkAction *action;
   gboolean enabled;
@@ -1789,22 +1797,25 @@ settings_changed_cb (GtkSettings *settings,
     name = NULL;
 
 #ifdef HAVE_CLUTTER
-  if (name == NULL || strcmp (name, "gtk-enable-animations") == 0) {
+  if (name == NULL || strcmp (name, "gtk-enable-animations") == 0)
     g_object_get (settings, "gtk-enable-animations", &enabled, NULL);
-
-    action = gtk_action_group_get_action (priv->action_group, "Animations");
-    gtk_action_set_visible (action, enabled);
-  }
+  else
 #endif /* HAVE_CLUTTER */
+    enabled = FALSE;
 
-#if defined(ENABLE_SOUND)
-  if (name == NULL || strcmp (name, "gtk-enable-event-sounds") == 0) {
+  action = gtk_action_group_get_action (priv->action_group, "Animations");
+  gtk_action_set_visible (action, enabled);
+
+#ifdef ENABLE_SOUND
+  if (name == NULL || strcmp (name, "gtk-enable-event-sounds") == 0)
     g_object_get (settings, "gtk-enable-event-sounds", &enabled, NULL);
-
-    action = gtk_action_group_get_action (priv->action_group, "Sound");
-    gtk_action_set_visible (action, enabled);
-  }
+  else
 #endif /* ENABLE_SOUND */
+    enabled = FALSE;
+
+  action = gtk_action_group_get_action (priv->action_group, "Sound");
+  gtk_action_set_visible (action, enabled);
+#endif /* HAVE_CLUTTER || ENABLE_SOUND */
 }
 
 static void
@@ -1812,6 +1823,7 @@ screen_changed_cb (GtkWidget *widget,
                    GdkScreen *previous_screen,
                    AisleriotWindow *window)
 {
+#if defined(HAVE_CLUTTER) || defined(ENABLE_SOUND)
   GdkScreen *screen;
   GtkSettings *settings;
 
@@ -1828,7 +1840,9 @@ screen_changed_cb (GtkWidget *widget,
   if (screen == NULL)
     return;
 
+#ifdef ENABLE_SOUND
   ar_sound_init (screen);
+#endif
 
   settings = gtk_widget_get_settings (widget);
   settings_changed_cb (settings, NULL, window);
@@ -1836,42 +1850,11 @@ screen_changed_cb (GtkWidget *widget,
   g_signal_connect (settings, "notify::gtk-enable-animations",
                     G_CALLBACK (settings_changed_cb), window);
 #endif
-#if defined (ENABLE_SOUND)
+#ifdef ENABLE_SOUND
   g_signal_connect (settings, "notify::gtk-enable-event-sounds",
                     G_CALLBACK (settings_changed_cb), window);
 #endif
-}
-
 #endif /* HAVE_CLUTTER || ENABLE_SOUND */
-
-/*
- * aisleriot_window_set_freecell_mode:
- * @window:
- *
- * Sets @window to FreeCell mode. In FreeCell mode,
- * the window is using the FreeCell variation, and doesn't allow
- * changing the game type.
- */
-static void
-aisleriot_window_set_freecell_mode (AisleriotWindow *window,
-                                    gboolean freecell_mode)
-{
-  AisleriotWindowPrivate *priv = window->priv;
-  GtkAction *action;
-
-  priv->freecell_mode = freecell_mode != FALSE;
-
-  if (freecell_mode) {
-    /* Inhibit game changing */
-    action = gtk_action_group_get_action (priv->action_group, "Select");
-    gtk_action_set_visible (action, FALSE);
-    action = gtk_action_group_get_action (priv->action_group, "RecentMenu");
-    gtk_action_set_visible (action, FALSE);
-
-    gtk_window_set_title (GTK_WINDOW (window), _("Freecell Solitaire"));
-  } else {
-    gtk_window_set_title (GTK_WINDOW (window), _("AisleRiot"));
-  }
 }
 
 static void
@@ -1919,7 +1902,7 @@ embed_size_allocate_cb (ArClutterEmbed *embed,
 
 /* Class implementation */
 
-G_DEFINE_TYPE (AisleriotWindow, aisleriot_window, GTK_TYPE_WINDOW)
+G_DEFINE_TYPE (AisleriotWindow, aisleriot_window, GTK_TYPE_APPLICATION_WINDOW)
 
 static void
 aisleriot_window_style_set (GtkWidget *widget,
@@ -2089,18 +2072,14 @@ aisleriot_window_init (AisleriotWindow *window)
       N_("Pick up and drop cards by clicking"),
       G_CALLBACK (clickmove_toggle_cb),
       FALSE /* not active by default */ },
-#ifdef ENABLE_SOUND
    { "Sound", NULL, N_("_Sound"), NULL,
       N_("Whether or not to play event sounds"),
       G_CALLBACK (sound_toggle_cb),
       FALSE /* not active by default */ },
-#endif /* ENABLE_SOUND */
-#ifdef HAVE_CLUTTER
    { "Animations", NULL, N_("_Animations"), NULL,
       N_("Whether or not to animate card moves"),
       G_CALLBACK (animations_toggle_cb),
       FALSE /* not active by default */ },
-#endif /* HAVE_CLUTTER */
   };
 
   static const char names[][16] = {
@@ -2115,88 +2094,6 @@ aisleriot_window_init (AisleriotWindow *window)
     "LeaveFullscreen",
   };
 
-  static const char ui_description[] =
-    "<ui>"
-      "<menubar name='MainMenu'>"
-        "<menu action='GameMenu'>"
-          "<menuitem action='NewGame'/>"
-          "<menuitem action='RestartGame'/>"
-          "<menuitem action='Statistics'/>"
-          "<menuitem action='Select'/>"
-          "<menu action='RecentMenu'/>"
-          "<separator/>"
-          "<menuitem action='CloseWindow'/>"
-        "</menu>"
-        "<menu action='ViewMenu'>"
-          "<menuitem action='Fullscreen'/>"
-          "<menuitem action='Toolbar'/>"
-          "<menuitem action='Statusbar'/>"
-          "<separator/>"
-          "<menu action='ThemeMenu'>"
-            "<placeholder name='ThemesPH'/>"
-            "<separator/>"
-            "<menuitem action='InstallThemes'/>"
-          "</menu>"
-        "</menu>"
-        "<menu action='ControlMenu'>"
-          "<menuitem action='UndoMove'/>"
-          "<menuitem action='RedoMove'/>"
-          "<menuitem action='Deal'/>"
-          "<menuitem action='Hint'/>"
-          "<separator/>"
-          "<menuitem action='ClickToMove'/>"
-#ifdef ENABLE_SOUND
-          "<menuitem action='Sound'/>"
-#endif
-#ifdef HAVE_CLUTTER
-          "<menuitem action='Animations'/>"
-#endif
-        "</menu>"
-        "<menu action='OptionsMenu'/>"
-        "<menu action='HelpMenu'>"
-          "<menuitem action='Contents'/>"
-          "<menuitem action='HelpGame'/>"
-          "<menuitem action='About'/>"
-        "</menu>"
-#ifdef ENABLE_DEBUG_UI
-        "<menu action='DebugMenu'>"
-          "<menuitem action='DebugTweakStyle'/>"
-          "<menuitem action='DebugTweakSettings'/>"
-          "<separator/>"
-          "<menuitem action='DebugChooseSeed'/>"
-          "<menuitem action='DebugMoveNextScreen'/>"
-          "<menuitem action='DebugDelayedMoveNextScreen'/>"
-          "<menuitem action='DebugException'/>"
-          "<separator/>"
-          "<menuitem action='DebugCycle'/>"
-          "<separator/>"
-          "<menuitem action='DebugGameFirst'/>"
-          "<menuitem action='DebugGamePrev'/>"
-          "<menuitem action='DebugGameNext'/>"
-          "<menuitem action='DebugGameLast'/>"
-        "</menu>"
-#endif /* ENABLE_DEBUG_UI */
-      "</menubar>"
-      "<toolbar name='Toolbar'>"
-        "<toolitem action='NewGame'/>"
-        "<toolitem action='RestartGame'/>"
-        "<toolitem action='Select'/>"
-        "<separator/>"
-        "<toolitem action='UndoMove'/>"
-        "<toolitem action='RedoMove'/>"
-        "<toolitem action='Deal'/>"
-        "<toolitem action='Hint'/>"
-        "<separator name='LeaveFullscreenSep' expand='true'/>"
-        "<toolitem action='LeaveFullscreen'/>"
-#ifdef ENABLE_DEBUG_UI
-        "<toolitem action='DebugGameFirst'/>"
-        "<toolitem action='DebugGamePrev'/>"
-        "<toolitem action='DebugGameNext'/>"
-        "<toolitem action='DebugGameLast'/>"
-#endif
-      "</toolbar>"
-    "</ui>";
-
   AisleriotWindowPrivate *priv;
   GtkWidget *main_vbox;
   GtkAccelGroup *accel_group;
@@ -2206,6 +2103,7 @@ aisleriot_window_init (AisleriotWindow *window)
   guint i;
   GtkStatusbar *statusbar;
   GtkWidget *statusbar_hbox, *label, *time_box;
+  GError *error = NULL;
 #ifdef HAVE_CLUTTER
   ClutterContainer *stage;
 #endif
@@ -2340,8 +2238,12 @@ aisleriot_window_init (AisleriotWindow *window)
   /* Load the UI after we've connected the statusbar,
    * otherwise not all actions will have statusbar help.
    */
-  gtk_ui_manager_add_ui_from_string (priv->ui_manager, ui_description,
-                                     strlen (ui_description), NULL);
+  gtk_ui_manager_add_ui_from_resource (priv->ui_manager, "/org/gnome/aisleriot/ui/menus.xml", &error);
+  g_assert_no_error (error);
+#ifdef ENABLE_DEBUG_UI
+  gtk_ui_manager_add_ui_from_resource (priv->ui_manager, "/org/gnome/aisleriot/ui/debug-menus.xml", &error);
+  g_assert_no_error (error);
+#endif /* ENABLE_DEBUG_UI */
 
   priv->main_menu = gtk_ui_manager_get_widget (priv->ui_manager, MAIN_MENU_UI_PATH);
   priv->toolbar = gtk_ui_manager_get_widget (priv->ui_manager, TOOLBAR_UI_PATH);
@@ -2530,23 +2432,6 @@ aisleriot_window_finalize (GObject *object)
   G_OBJECT_CLASS (aisleriot_window_parent_class)->finalize (object);
 }
 
-static void
-aisleriot_window_set_property (GObject      *object,
-                               guint         property_id,
-                               const GValue *value,
-                               GParamSpec   *pspec)
-{
-  AisleriotWindow *window = AISLERIOT_WINDOW (object);
-
-  switch (property_id) {
-    case PROP_FREECELL_MODE:
-      aisleriot_window_set_freecell_mode (window, g_value_get_boolean (value));
-      break;
-
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
-  }
-}
 static void
 aisleriot_window_class_init (AisleriotWindowClass *klass)
 {
@@ -2555,26 +2440,11 @@ aisleriot_window_class_init (AisleriotWindowClass *klass)
 
   gobject_class->dispose = aisleriot_window_dispose;
   gobject_class->finalize = aisleriot_window_finalize;
-  gobject_class->set_property = aisleriot_window_set_property;
 
   widget_class->window_state_event = aisleriot_window_state_event;
   widget_class->style_set = aisleriot_window_style_set;
 
   g_type_class_add_private (gobject_class, sizeof (AisleriotWindowPrivate));
-
-  /**
-   * AisleriotWindow:freecell-mode:
-   *
-   * Whether the window is in freecell mode.
-   */
-  g_object_class_install_property
-    (gobject_class,
-     PROP_FREECELL_MODE,
-     g_param_spec_boolean ("freecell-mode", NULL, NULL,
-                           FALSE,
-                           G_PARAM_WRITABLE |
-                           G_PARAM_CONSTRUCT_ONLY |
-                           G_PARAM_STATIC_STRINGS));
 }
 
 /* public API */
@@ -2585,13 +2455,39 @@ aisleriot_window_class_init (AisleriotWindowClass *klass)
  * Returns: a new #AisleriotWindow
  */
 GtkWidget *
-aisleriot_window_new (gboolean freecell_mode)
+aisleriot_window_new (GtkApplication *application)
 {
   return g_object_new (AISLERIOT_TYPE_WINDOW,
-                       "freecell-mode", freecell_mode,
+                       "application", application,
+                       "show-menubar", FALSE,
                        NULL);
 }
 
+/**
+ * aisleriot_window_get_ui_manager:
+ * @window:
+ *
+ * Returns: (transfer none): the window's #GtkUIManager
+ */
+GtkUIManager *
+aisleriot_window_get_ui_manager (AisleriotWindow *window)
+{
+  return window->priv->ui_manager;
+}
+
+/**
+ * aisleriot_window_get_ui_manager:
+ * @window:
+ *
+ * Returns: (transfer none): the window's #GtkUIManager
+ */
+GtkAction *
+aisleriot_window_get_action (AisleriotWindow *window,
+                             const char *action_name)
+{
+  return gtk_action_group_get_action (window->priv->action_group, action_name);
+}
+
 typedef struct {
   AisleriotWindow *window;
   char *game_module;
@@ -2615,6 +2511,7 @@ load_idle_cb (LoadIdleData *data)
   AisleriotWindowPrivate *priv = data->window->priv;
   GError *error = NULL;
   GRand *rand;
+  char *pref;
 
   if (!aisleriot_game_load_game (priv->game, data->game_module, &error)) {
     GtkWidget *dialog;
@@ -2630,26 +2527,19 @@ load_idle_cb (LoadIdleData *data)
                                      name);
     g_free (name);
 
-    if (priv->freecell_mode ||
-        strcmp (data->game_module, DEFAULT_VARIATION) == 0) {
-      /* Loading freecell/the fallback game failed; all we can do is exit */
-      g_signal_connect_swapped (dialog, "response",
-                                G_CALLBACK (gtk_widget_destroy), data->window);
-    } else {
-      gtk_message_dialog_format_secondary_text
-        (GTK_MESSAGE_DIALOG (dialog),
-         "%s\n\n%s",
-         _("Aisleriot cannot find the last game you played."),
-         _("This usually occurs when you run an older version of Aisleriot "
-           "which does not have the game you last played. "
-           "The default game, Klondike, is being started instead."));
+    gtk_message_dialog_format_secondary_text
+      (GTK_MESSAGE_DIALOG (dialog),
+       "%s\n\n%s",
+       _("Aisleriot cannot find the last game you played."),
+       _("This usually occurs when you run an older version of Aisleriot "
+         "which does not have the game you last played. "
+         "The default game, Klondike, is being started instead."));
 
-      /* FIXME: add @error->message to a textview in a Detailed… expander */
-      g_printerr ("Scheme exception:\n-- 8< --\n%s\n-- >8 --\n", error->message);
+    /* FIXME: add @error->message to a textview in a Detailed… expander */
+    g_printerr ("Scheme exception:\n-- 8< --\n%s\n-- >8 --\n", error->message);
 
-      g_signal_connect (dialog, "response",
-                        G_CALLBACK (load_error_response_cb), data->window);
-    }
+    g_signal_connect (dialog, "response",
+                      G_CALLBACK (load_error_response_cb), data->window);
 
     g_error_free (error);
 
@@ -2659,15 +2549,11 @@ load_idle_cb (LoadIdleData *data)
   }
 
   /* Now that we know we can successfully load this variation,
-   * store it in conf, except when we're running in freecell mode.
+   * store it in conf
    */
-  if (!priv->freecell_mode) {
-    char *pref;
-
-    pref = g_strconcat (data->game_module, ".scm", NULL);
-    ar_conf_set_string (NULL, aisleriot_conf_get_key (CONF_VARIATION), pref);
-    g_free (pref);
-  }
+  pref = g_strconcat (data->game_module, ".scm", NULL);
+  ar_conf_set_string (NULL, aisleriot_conf_get_key (CONF_VARIATION), pref);
+  g_free (pref);
 
   rand = data->rand;
   data->rand = NULL;
